#ifndef MOTIONCONTROL_H
#define MOTIONCONTROL_H

#include <Core/statechart/Statechart.h>
#include <core/RobotStatechartContext.h>

namespace armarx
{

namespace MoveHelpers
{
    /**
    *   MotionControl contains convenience functions that allow to move parts of the robot,
    *   i.e. direct and inverse kinematics for arbitrary kinematic chains and predefined
    *   parts like the arms.
    */
    class MotionControl : public RemoteStateHandler<RobotStatechartContext>
    {
    public:
        MotionControl();
        void onInitRemoteStateHandler();
        void onConnectRemoteStateHandler();
        std::string getDefaultName() const { return "MotionControl"; }
    };


    DEFINEEVENT(EvJointTargetReached)
    DEFINEEVENT(EvJointVelocitiesTargetReached)
    DEFINEEVENT(EvCalculationDone)
    DEFINEEVENT(EvTimeout)


    struct MotionControlTestState : StateTemplate<MotionControlTestState>
    {
        void defineParameters();
        void defineSubstates();
    };


    /**
    *   Move the joints of a kinematic chain to the desired values.
    *   Parameters?
    */
    struct MoveJoints : StateTemplate<MoveJoints>
    {
        RobotStatechartContext* context;
        ConditionIdentifier targetReachedCondition;
        ActionEventIdentifier timeoutEvent;
        MoveJoints();
        void defineParameters();
        void onEnter();
        void onBreak();
        void onExit();
    };


    /**
    *   Move the joints of a kinematic chain to the desired values using velocity control.
    *   Parameters?
    */
    struct MoveJointsVelocityControl : StateTemplate<MoveJointsVelocityControl>
    {
        RobotStatechartContext* context;
        ConditionIdentifier targetVelocitiesReachedCondition;
        ActionEventIdentifier timeoutEvent;
        void defineParameters();
        void onEnter();
        void onExit();
    };


    /**
    *   Move the TCP to a desired pose.
    *   Parameters?
    */
    struct MoveTCPPoseIK : StateTemplate<MoveTCPPoseIK>
    {
        RobotStatechartContext* context;
        ConditionIdentifier targetReachedCondition;
        ActionEventIdentifier timeoutEvent;
        void defineParameters();
        void defineSubstates();
        void onEnter();
        void onExit();
    };


    DEFINEEVENT(EvLastPointReached)

    /**
    *   Move the TCP along a trajectory of poses.
    *   Parameters?
    */
    struct MoveTCPTrajectory : StateTemplate<MoveTCPTrajectory>
    {
        RobotStatechartContext* context;
        ConditionIdentifier lastTargetReachedCondition;
        ActionEventIdentifier timeoutEvent;
        ActionEventIdentifier trajectoryPointCounterID;
        void defineParameters();
        void defineSubstates();
        void onEnter();
        void onExit();
    };



    struct MoveTCPTrajectoryNextPoint : StateTemplate<MoveTCPTrajectoryNextPoint>
    {
        RobotStatechartContext* context;
        void defineParameters();
        void defineSubstates();
        void onEnter();
        void onExit();
    };




    /**
    *   Stop all motion of the robot.
    *   Parameters?
    */
    struct StopRobot : StateTemplate<StopRobot>
    {
        RobotStatechartContext* context;

        void onEnter();
    };



    /////////////////////////////////////////////////////////////////////
    //// intermediate states
    /////////////////////////////////////////////////////////////////////

    struct CalculateJointAngleConfiguration : StateTemplate<CalculateJointAngleConfiguration>
    {

        ActionEventIdentifier timeoutEvent;
        void defineParameters();
        void run();

    };

    struct ValidateJointAngleConfiguration : StateTemplate<ValidateJointAngleConfiguration>
    {
        RobotStatechartContext* context;
        ActionEventIdentifier timeoutEvent;
        void defineParameters();
        void onEnter();
    };

} // namespace MoveHelpers

} // namespace armarx

#endif // MOTIONCONTROL_H
